"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

Object.defineProperty(exports, "__esModule", { value: true });
var power_di_1 = require("power-di");
var utils_1 = require("power-di/utils");
var Odux_1 = require("./Odux");
var StoreAdapter_1 = require("./StoreAdapter");
var MetaSymbol = Symbol('MetaSymbol');
function getStoreMeta(store) {
    var meta = store[MetaSymbol];
    if (!meta) {
        Object.defineProperty(store, MetaSymbol, {
            enumerable: false,
            writable: false,
            value: {}
        });
        return getStoreMeta(store);
    }
    return meta;
}
exports.getStoreMeta = getStoreMeta;

var BaseStore = function () {
    function BaseStore(storeOrIoC) {
        _classCallCheck(this, BaseStore);

        var odux = storeOrIoC instanceof power_di_1.IocContext ? storeOrIoC.get(Odux_1.Odux) : storeOrIoC;
        var type = utils_1.getGlobalType(this.constructor);
        if (!odux) {
            throw new Error("registerStore [" + type + " (alias: " + this.$aliasName + ")] fail! no odux.");
        }
        function getDataType(target, p) {
            if (BaseStore.excludeKeys.indexOf(p) >= 0) {
                return false;
            }
            var value = target[p];
            var type = typeof value === "undefined" ? "undefined" : _typeof(value);
            if (type === 'function') return 'function';
            return ['undefined', 'number', 'string', 'bool', 'object', 'symbol'].indexOf(type) >= 0 ? 'data' : false;
        }
        var thisProxy = new Proxy(this, {
            get: function get(target, p) {
                var dataType = getDataType(target, p);
                var meta = getStoreMeta(target);
                return dataType === 'data' ? meta.adapter.getData()[p] : dataType === 'function' ? meta.adapter.transactionChange(target[p].bind(thisProxy)) : target[p];
            },
            set: function set(target, p, value) {
                if (getDataType(target, p) === 'data') {
                    thisProxy.changeData(function () {
                        var data = getStoreMeta(target).adapter.getData();
                        data[p] = value;
                    });
                } else {
                    target[p] = value;
                }
                return true;
            }
        });
        var meta = getStoreMeta(this);
        meta.adapter = new StoreAdapter_1.StoreAdapter(odux, type);
        odux.registerStore(this);
        return thisProxy;
    }

    _createClass(BaseStore, [{
        key: "changeData",

        /** 数据变更方法，推荐把业务逻辑封装在 Store 上独立的方法内 */
        value: function changeData(func) {
            func();
        }
        /** apply change immediately */

    }, {
        key: "applyChange",
        value: function applyChange() {
            var meta = getStoreMeta(this);
            meta.adapter.applyChange();
        }
    }, {
        key: "$aliasName",
        set: function set(value) {
            getStoreMeta(this).adapter.setAliasName(value);
        }
    }, {
        key: "$prefix",
        set: function set(value) {
            getStoreMeta(this).adapter.setPrefix(value);
        }
    }]);

    return BaseStore;
}();

BaseStore.excludeKeys = [MetaSymbol, '$aliasName', '$prefix', 'applyChange'].concat(_toConsumableArray(Object.getOwnPropertyNames(Object)));
exports.BaseStore = BaseStore;
//# sourceMappingURL=BaseStore.js.map