"use strict";

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

Object.defineProperty(exports, "__esModule", { value: true });
var BaseEvent_1 = require("./BaseEvent");
var utils_1 = require("power-di/utils");

var StoreChangeEvent = function (_BaseEvent_1$BaseEven) {
    _inherits(StoreChangeEvent, _BaseEvent_1$BaseEven);

    function StoreChangeEvent() {
        _classCallCheck(this, StoreChangeEvent);

        return _possibleConstructorReturn(this, (StoreChangeEvent.__proto__ || Object.getPrototypeOf(StoreChangeEvent)).apply(this, arguments));
    }

    return StoreChangeEvent;
}(BaseEvent_1.BaseEvent);

exports.StoreChangeEvent = StoreChangeEvent;
var getUid = function () {
    var id = 0;
    return function () {
        return id++;
    };
}();

var EventBus = function () {
    function EventBus() {
        _classCallCheck(this, EventBus);

        this.eventHandlers = {};
        this.componentListeners = [];
    }

    _createClass(EventBus, [{
        key: "setComponentListener",
        value: function setComponentListener(cId, listener, storeKeys) {
            var clIndex = this.componentListeners.findIndex(function (c) {
                return c.cId === cId;
            });
            if (!listener && clIndex >= 0) {
                this.componentListeners.splice(clIndex, 1);
                return;
            }
            var cl = this.componentListeners[clIndex];
            if (!cl) {
                if (!cId) {
                    cId = "$odux$cl_" + getUid();
                }
                this.componentListeners.push({ cId: cId, listener: listener, storeKeys: storeKeys });
            } else {
                cl.listener = listener;
                cl.storeKeys = storeKeys;
            }
            return cId;
        }
    }, {
        key: "emitComponentEvent",
        value: function emitComponentEvent(evt) {
            var storeKeys = evt.data.map(function (e) {
                return e.storeKey;
            }).filter(function (s) {
                return s;
            });
            var components = storeKeys.length ? this.componentListeners.filter(function (cl) {
                return !cl.storeKeys || storeKeys.some(function (sk) {
                    return cl.storeKeys.indexOf(sk) >= 0;
                });
            }) : this.componentListeners;
            components.forEach(function (cl) {
                cl.listener(evt);
            });
        }
        /**
         * addEventListener
         * @param eventType typeof BaseEvent
         * @param callback event callback
         */

    }, {
        key: "addListener",
        value: function addListener(eventType, callback) {
            var type = this.getEventType(eventType);
            if (!this.eventHandlers[type]) {
                this.eventHandlers[type] = [];
            }
            this.eventHandlers[type].push(callback);
        }
        /**
         * removeEventListener
         * @param eventType typeof BaseEvent
         * @param callback event callback
         */

    }, {
        key: "removeListener",
        value: function removeListener(eventType, callback) {
            var type = this.getEventType(eventType);
            var element = this.eventHandlers[type];
            if (element) {
                var index = element.indexOf(callback);
                if (index !== -1) {
                    element.splice(index, 1);
                }
            }
        }
    }, {
        key: "removeAllListeners",
        value: function removeAllListeners(eventType) {
            var type = this.getEventType(eventType);
            this.eventHandlers[type] = [];
        }
    }, {
        key: "emit",
        value: function emit(event) {
            if (!(event instanceof BaseEvent_1.BaseEvent)) {
                throw new Error("need instance of 'BaseEvent', but " + (typeof event === "undefined" ? "undefined" : _typeof(event)) + ".");
            }
            var type = this.getEventType(event.constructor);
            this.eventHandlers[type] && this.eventHandlers[type].forEach(function (handler) {
                handler(event);
            });
        }
    }, {
        key: "getEventType",
        value: function getEventType(evt) {
            try {
                return utils_1.getGlobalType(evt);
            } catch (error) {
                throw new Error("EventType NotFound. " + error);
            }
        }
    }]);

    return EventBus;
}();

exports.EventBus = EventBus;
//# sourceMappingURL=EventBus.js.map